@@ -40,9 +40,13 @@ Symbol::Symbol(const u1* name, int length, int refcount) {
   _refcount = refcount;
   _length = length;
   _identity_hash = (short)os::random();
-  for (int i = 0; i < _length; i++) {
-    byte_at_put(i, name[i]);
-  }
+  // _body[0..1] are allocated in the header just by coincidence in the current
+  // implementation of Symbol. They are read by identity_hash(), so make sure they
+  // are initialized.
+  // No other code should assume that _body[0..1] are always allocated. E.g., do
+  // not unconditionally read base()[0] as that will be invalid for an empty Symbol.
+  _body[0] = _body[1] = 0;
+  memcpy(_body, name, length);
 }
 
 void* Symbol::operator new(size_t sz, int len, TRAPS) throw() {
@@ -266,7 +270,7 @@ void Symbol::print_as_signature_external_parameters(outputStream *os) {
 unsigned int Symbol::new_hash(juint seed) {
   ResourceMark rm;
   // Use alternate hashing algorithm on this symbol.
-  return AltHashing::murmur3_32(seed, (const jbyte*)as_C_string(), utf8_length());
+  return AltHashing::halfsiphash_32(seed, (const uint8_t*)as_C_string(), utf8_length());
 }
 
 void Symbol::increment_refcount() {
