@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1994, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1994, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -56,6 +56,8 @@ Java_java_lang_System_identityHashCode(JNIEnv *env, jobject this, jobject x)
     return JVM_IHashCode(env, x);
 }
 
+/* VENDOR, VENDOR_URL, VENDOR_URL_BUG are set in VersionProps.java.template. */
+
 #define PUTPROP(props, key, val)                                     \
     if (1) {                                                         \
         jstring jkey, jval;                                          \
@@ -108,27 +110,12 @@ Java_java_lang_System_identityHashCode(JNIEnv *env, jobject this, jobject x)
         (*env)->DeleteLocalRef(env, jkey);                            \
     } else ((void) 0)
 
-/* Third party may overwrite these values. */
-#ifndef VENDOR
-#define VENDOR "Oracle Corporation"
-#endif
-#ifndef VENDOR_URL
-#define VENDOR_URL "http://java.oracle.com/"
-#endif
-#ifndef VENDOR_URL_BUG
-#define VENDOR_URL_BUG "http://bugreport.java.com/bugreport/"
-#endif
-
-#define JAVA_MAX_SUPPORTED_VERSION 54
-#define JAVA_MAX_SUPPORTED_MINOR_VERSION 0
-
 #ifdef JAVA_SPECIFICATION_VENDOR /* Third party may NOT overwrite this. */
   #error "ERROR: No override of JAVA_SPECIFICATION_VENDOR is allowed"
 #else
   #define JAVA_SPECIFICATION_VENDOR "Oracle Corporation"
 #endif
 
-static int fmtdefault; // boolean value
 jobject fillI18nProps(JNIEnv *env, jobject props, char *baseKey,
                       char *platformDispVal, char *platformFmtVal,
                       jmethodID putID, jmethodID getPropID) {
@@ -144,16 +131,9 @@ jobject fillI18nProps(JNIEnv *env, jobject props, char *baseKey,
         const char *baseVal = "";
 
         /* user.xxx base property */
-        if (fmtdefault) {
-            if (platformFmtVal) {
-                PUTPROP(props, baseKey, platformFmtVal);
-                baseVal = platformFmtVal;
-            }
-        } else {
-            if (platformDispVal) {
-                PUTPROP(props, baseKey, platformDispVal);
-                baseVal = platformDispVal;
-            }
+        if (platformDispVal) {
+            PUTPROP(props, baseKey, platformDispVal);
+            baseVal = platformDispVal;
         }
 
         /* user.xxx.display property */
@@ -217,18 +197,15 @@ Java_java_lang_System_initProperties(JNIEnv *env, jclass cla, jobject props)
 
     PUTPROP(props, "java.specification.version",
             VERSION_SPECIFICATION);
+    PUTPROP(props, "java.specification.maintenance.version",
+            "2");
     PUTPROP(props, "java.specification.name",
             "Java Platform API Specification");
     PUTPROP(props, "java.specification.vendor",
             JAVA_SPECIFICATION_VENDOR);
 
-    PUTPROP(props, "java.version", VERSION_SHORT);
-    PUTPROP(props, "java.vendor", VENDOR);
-    PUTPROP(props, "java.vendor.url", VENDOR_URL);
-    PUTPROP(props, "java.vendor.url.bug", VENDOR_URL_BUG);
-
-    jio_snprintf(buf, sizeof(buf), "%d.%d", JAVA_MAX_SUPPORTED_VERSION,
-                                            JAVA_MAX_SUPPORTED_MINOR_VERSION);
+    jio_snprintf(buf, sizeof(buf), "%d.%d", JVM_CLASSFILE_MAJOR_VERSION,
+                                            JVM_CLASSFILE_MINOR_VERSION);
     PUTPROP(props, "java.class.version", buf);
 
     if (sprops->awt_toolkit) {
@@ -255,30 +232,14 @@ Java_java_lang_System_initProperties(JNIEnv *env, jclass cla, jobject props)
     PUTPROP(props, "line.separator", sprops->line_separator);
 
     /*
-     *  user.language
-     *  user.script, user.country, user.variant (if user's environment specifies them)
-     *  file.encoding
-     *  file.encoding.pkg
+     * file encoding for stdout and stderr
      */
-    PUTPROP(props, "user.language", sprops->language);
-    if (sprops->script) {
-        PUTPROP(props, "user.script", sprops->script);
-    }
-    if (sprops->country) {
-        PUTPROP(props, "user.country", sprops->country);
-    }
-    if (sprops->variant) {
-        PUTPROP(props, "user.variant", sprops->variant);
-    }
-    PUTPROP(props, "file.encoding", sprops->encoding);
-    PUTPROP(props, "sun.jnu.encoding", sprops->sun_jnu_encoding);
     if (sprops->sun_stdout_encoding != NULL) {
         PUTPROP(props, "sun.stdout.encoding", sprops->sun_stdout_encoding);
     }
     if (sprops->sun_stderr_encoding != NULL) {
         PUTPROP(props, "sun.stderr.encoding", sprops->sun_stderr_encoding);
     }
-    PUTPROP(props, "file.encoding.pkg", "sun.io");
 
     /* unicode_encoding specifies the default endianness */
     PUTPROP(props, "sun.io.unicode.encoding", sprops->unicode_encoding);
@@ -328,8 +289,9 @@ Java_java_lang_System_initProperties(JNIEnv *env, jclass cla, jobject props)
 #endif
 
     /* !!! DO NOT call PUTPROP_ForPlatformNString before this line !!!
-     * !!! I18n properties have not been set up yet !!!
+     * !!! The platform native encoding for strings has not been set up yet !!!
      */
+    InitializeEncoding(env, sprops->sun_jnu_encoding);
 
     /* Printing properties */
     /* Note: java.awt.printerjob is an implementation private property which
@@ -392,29 +354,8 @@ Java_java_lang_System_initProperties(JNIEnv *env, jclass cla, jobject props)
         PUTPROP(props, "sun.desktop", sprops->desktop);
     }
 
-    /*
-     * unset "user.language", "user.script", "user.country", and "user.variant"
-     * in order to tell whether the command line option "-DXXXX=YYYY" is
-     * specified or not.  They will be reset in fillI18nProps() below.
-     */
-    REMOVEPROP(props, "user.language");
-    REMOVEPROP(props, "user.script");
-    REMOVEPROP(props, "user.country");
-    REMOVEPROP(props, "user.variant");
-    REMOVEPROP(props, "file.encoding");
-
     ret = JVM_InitProperties(env, props);
 
-    /* Check the compatibility flag */
-    GETPROP(props, "sun.locale.formatasdefault", jVMVal);
-    if (jVMVal) {
-        const char * val = (*env)->GetStringUTFChars(env, jVMVal, 0);
-        CHECK_NULL_RETURN(val, NULL);
-        fmtdefault = !strcmp(val, "true");
-        (*env)->ReleaseStringUTFChars(env, jVMVal, val);
-        (*env)->DeleteLocalRef(env, jVMVal);
-    }
-
     /* reconstruct i18n related properties */
     fillI18nProps(env, props, "user.language", sprops->display_language,
         sprops->format_language, putID, getPropID);
@@ -433,16 +374,15 @@ Java_java_lang_System_initProperties(JNIEnv *env, jclass cla, jobject props)
          */
         PUTPROP(props, "file.encoding", sprops->encoding);
 #else
-        if (fmtdefault) {
-            PUTPROP(props, "file.encoding", sprops->encoding);
-        } else {
-            PUTPROP(props, "file.encoding", sprops->sun_jnu_encoding);
-        }
+        PUTPROP(props, "file.encoding", sprops->sun_jnu_encoding);
 #endif
     } else {
         (*env)->DeleteLocalRef(env, jVMVal);
     }
 
+    // Platform defined encoding properties override any on the command line
+    PUTPROP(props, "sun.jnu.encoding", sprops->sun_jnu_encoding);
+
     return ret;
 }
 
