@@ -24,8 +24,7 @@
 /**
  * @test
  * @summary Test deadlock detection
- * @requires vm.hasSAandCanAttach
- * @requires os.family != "mac"
+ * @requires vm.hasSA
  * @library /test/lib
  * @modules java.base/jdk.internal.misc
  * @modules java.management
@@ -38,19 +37,17 @@ import java.util.stream.Collectors;
 
 import jdk.test.lib.apps.LingeredApp;
 import jdk.test.lib.apps.LingeredAppWithDeadlock;
-
-import jdk.test.lib.Utils;
-import jdk.test.lib.Platform;
 import jdk.test.lib.JDKToolLauncher;
 import jdk.test.lib.process.OutputAnalyzer;
 import jdk.test.lib.process.ProcessTools;
+import jdk.test.lib.SA.SATestUtils;
+import jdk.test.lib.Utils;
 
 import jtreg.SkippedException;
 
 public class DeadlockDetectionTest {
 
     private static LingeredAppWithDeadlock theApp = null;
-    private static ProcessBuilder processBuilder = new ProcessBuilder();
 
     private static OutputAnalyzer jstack(String... toolArgs) throws Exception {
         JDKToolLauncher launcher = JDKToolLauncher.createUsingTestJDK("jhsdb");
@@ -61,7 +58,7 @@ public class DeadlockDetectionTest {
             }
         }
 
-        processBuilder.command(launcher.getCommand());
+        ProcessBuilder processBuilder = SATestUtils.createProcessBuilder(launcher);
         System.out.println(processBuilder.command().stream().collect(Collectors.joining(" ")));
         OutputAnalyzer output = ProcessTools.executeProcess(processBuilder);
         System.out.println(output.getOutput());
@@ -70,6 +67,7 @@ public class DeadlockDetectionTest {
     }
 
     public static void main(String[] args) throws Exception {
+        SATestUtils.skipIfCannotAttach(); // throws SkippedException if attach not expected to work.
         System.out.println("Starting DeadlockDetectionTest");
 
         if (!LingeredApp.isLastModifiedWorking()) {
@@ -94,7 +92,6 @@ public class DeadlockDetectionTest {
                 output.shouldHaveExitValue(0);
                 output.shouldContain("Found a total of 1 deadlock.");
             }
-
         } finally {
             LingeredApp.stopApp(theApp);
         }
