@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @bug 8144903 8177466 8191842 8211694
+ * @bug 8144903 8177466 8191842 8211694 8213725 8239536
  * @summary Tests for EvaluationState.variables
  * @library /tools/lib
  * @modules jdk.compiler/com.sun.tools.javac.api
@@ -466,6 +466,15 @@ public class VariablesTest extends KullaTesting {
         assertVarDisplayName("var v6 = new Runnable() { public void run() { } };", "<anonymous class implementing Runnable>");
     }
 
+    public void varType() {
+        assertEval("import java.util.*;");
+        var firstVar = varKey(assertEval("var v1 = List.of(1);", added(VALID)));
+        assertEval("import list.List;", DiagCheck.DIAG_OK, DiagCheck.DIAG_ERROR, added(VALID),
+                                        ste(firstVar, VALID, RECOVERABLE_NOT_DEFINED, true, MAIN_SNIPPET));
+        assertEval("var v2 = java.util.List.of(1);", added(VALID));
+        assertEval("v2", "[1]");
+    }
+
     public void varDeclNoInit() {
         assertVarDeclNoInit("byte", "b",  "0");
         assertVarDeclNoInit("short", "h",  "0");
@@ -492,6 +501,21 @@ public class VariablesTest extends KullaTesting {
         assertVarDeclRedefNoInit("String", "s", "\"hi\"", "null");
     }
 
+    public void badPkgVarDecl() {
+        Compiler compiler = new Compiler();
+        Path nopkgdirpath = Paths.get("cp", "xyz");
+        compiler.compile(nopkgdirpath,
+                "public class TestZ { public static int V = 0; }\n");
+        assertDeclareFail("import static xyz.TestZ.V;",
+                        "compiler.err.cant.access");
+
+
+        VarSnippet v1 = varKey(assertEval("var v = xyz.TestZ.V;", IGNORE_VALUE, null,
+                DiagCheck.DIAG_ERROR, DiagCheck.DIAG_OK, added(RECOVERABLE_NOT_DEFINED)));
+        assertVariableDeclSnippet(v1, "v", "java.lang.Object", RECOVERABLE_NOT_DEFINED, SubKind.VAR_DECLARATION_WITH_INITIALIZER_SUBKIND, 0, 1);
+        assertEval("1+1", "2");
+    }
+
     private void assertVarDeclRedefNoInit(String typeName, String name, String value, String dvalue) {
         assertVarDeclRedefNoInit(typeName, name, value, value, dvalue);
     }
@@ -564,6 +588,10 @@ public class VariablesTest extends KullaTesting {
                 "\n" +
                 "public interface J<T> {\n" +
                 "   public List<T> get();\n" +
+                "}\n",
+                "package list;\n" +
+                "\n" +
+                "public class List {\n" +
                 "}\n");
         String tpath = compiler.getPath(path).toString();
         setUp(b -> b
